(*****************************************************

 Anti-Aliasing Line drawing Demo
 Copyright (c) 2001 by the author Jonas Widarsson

 Target:
   WIN32 GUI application

 Development environment:
   TMT Pascal Multitarget 3.50 SP2

 Release date:
   2001 september 2

 Purpose of this source code release:
   It is nice to draw beutiful lines in the client
   area of a standard application window.
   Since others might find my lines nice as well,
   the source code is released to make the crowds
   happy :)

 If you like these routines and would like to use
 them in your own programs, just let me know about
 it and I'll be satisfied.

 email:
  jonaswidarsson@home.se

 Please, DO report bugs!

******************************************************)

{$ifndef __WIN32__}
{$define INVALID_TARGET}
{$endif}
{$ifndef __GUI__}
{$define INVALID_TARGET}
{$endif}
{$ifdef INVALID_TARGET}
  This is a Win32 GUI application , please change target...
{$endif}


program AALine;
uses ShellAPI,windows,messages,CommCtrl;


{$r aal.res}

var   rect                                            : TRect;
      a,q                                             : Dword;
      S,S1                                            : String;
      Banner_BMP                                      : hBitmap;

      AppWnd, Btn_Banner, Track_AAG,
      Track_Mag, TextLabel                            : hWnd;

      Msg                                             : TMsg;
      Accelerators                                    : array [0..1] of TAccel;
      Acceleratortable                                : hAccel;

      WrkArea                                         : TRect;
      DC, MemDC                                       : HDC;
      MemBMP                                          : hBitmap;
      BITMAPINFO                                      : TBitmapInfo;
      DIBBits                                         : Pointer;

      MemBMPW                                         : Dword;
      MemBMPH                                         : Dword;


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Graphics functions  ////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

/*
  All Graphics functions declared here depend on correct values of three variables:

  * MemBMPW       ...the width of the Device Independent Bitmap where the pixels are located.
  * MemBMPH       ...and its height.
  * DIBBits       ...a pointer to the first pixel of the Device Independent Bitmap.

  These variables belongs to the Device Independent Bitmap (DIB) created in the function "InitMemory".
  The Bitmap is initialized with the width and height of the desktop...
  Of course that might consume a lot of memory.
  In my case, at least: (1280 * 1024 * 3 ) = 3 932 160 bytes
  The "3" is derived from the color resolution, 24-bit = 3 bytes per pixel.

  The 24-bit color resolution has nothing to do with your graphics card's features.
  This is only choosen to allow the DIB to contain nice color gradients.
  The memory consumption (probably) only affects your system RAM.

  Implementing one's own graphics routines for a DIB like it is done here prooves
  to be a practical way, since all pixels are located in a single flat memory area.
  It is possible to do things the way we used to when we were hacking in mode 13h!

  When all imminent drawing operations are performed we just "bitblt"
  (see Windows API docs) the works to the application window's device context.
  How quickly that is done is totally dependent on
  your graphics board's 2D-acceleration features.

  The "Main" section of the source code is just windows-related code.
  The purpose of this file is contained in the "Graphics" section.

  So... Have a nice day.

  I wish TMT Development Corporation only good things, since they have provided
  us the possibility to create quickly executing programs in our favourite language:

  Pascal

  Best wishes
  Jonas Widarsson
  Halsingland, Sweden

*/

Procedure InitMemory;
 begin
  DC:=GetDC(AppWnd);
  MemDC  := CreateCompatibleDC(DC);
  GetWindowRect(GetDesktopWindow,rect);
  with BITMAPINFO.bmiHeader do
   begin
    biSize           :=SizeOf(BITMAPINFO.bmiHeader);
    biWidth          :=rect.right;
    biHeight         :=-1 * rect.bottom;
    biPlanes         :=1;
    biBitCount       :=24;
    biCompression    :=BI_RGB ;
    biSizeImage      :=0;
    biXPelsPerMeter  :=10;
    biYPelsPerMeter  :=10;
    biClrUsed        :=0;
    biClrImportant   :=0;
   end;
  MemBMPW := BITMAPINFO.bmiHeader.biWidth ;
  MemBMPH := Abs(BITMAPINFO.bmiHeader.biHeight);
  FillChar(BITMAPINFO.bmiColors,SizeOf(BITMAPINFO.bmiColors),0);
  MemBMP := CreateDIBSection( MemDC, BITMAPINFO, DIB_RGB_COLORS, DIBBits, 0, 0 );
  SelectObject(MemDC,MemBMP);
 end;

Function  GetPixelXY( x1,y1 : Longint): Colorref;//Strength:= 0 = transparent   //Strength:= 255 = opaque
var r,g,b:Byte;
    MemBMPOffset : Pointer;
begin  // This function is only used by:   Procedure Magnify(x1,y1:Longint);
 asm
  mov eax, MemBMPW
  mov ebx, y1
  mul ebx
  add eax, x1
  mov MemBMPOffset,eax

  mov eax,MemBMPW
  mov ebx,MemBMPH
  mul ebx
  mov ebx,MemBMPOffset
  cmp eax,ebx
  jng @done1
  xor ebx,07fffffffh
  cmp ebx,000000000h
  jng @done1

  mov eax,DIBBits
  mov esi,eax  // Bitmap memory address into esi

  //Simply put pixel without fading to background color
  mov eax,MemBMPOffset
  mov ebx, 3        //24-bit color
  mul ebx
  add esi, eax

  //Pixel located
  mov bl,[esi]
//  mov b,bl
  shl ebx,16
  // blue read
  mov bl,[esi+2]
  mov r,bl
  // red read
  mov bh,[esi+1]
  mov g,bl
  mov result,ebx

  // green read
  jmp @done2
@done1:
  mov result,0
@done2:
 end;
end;

Procedure   PutPixelXY( x1,y1 : Longint; Color : Colorref);assembler;//Strength:= 0 = transparent   //Strength:= 255 = opaque
var r,g,b:Byte;
    MemBMPOffset : Pointer;
 asm // This function is only used by:   Procedure Magnify(x1,y1:Longint);
  mov eax, MemBMPW
  mov ebx, y1
  mul ebx
  add eax, x1
  mov MemBMPOffset,eax

  mov eax,MemBMPW
  mov ebx,MemBMPh
  mul ebx
  mov ebx,MemBMPOffset
  cmp eax,ebx
  jng @done
  xor ebx,07fffffffh
  cmp ebx,000000000h
  jng @done

  mov eax,DIBBits
  mov EDI,eax  // Bitmap memory address into EDI

  mov eax,MemBMPOffset
  mov ebx, 3        //24-bit color
  mul ebx
  add edi, eax

  //Pixel located
  Mov ebx,Color  // Store color values in variables

  mov [edi+2],bl
  // red written
  mov [edi+1],bh
  // green written
  shr ebx,16
  mov [edi],bl
  // blue written
@done:
end;

Procedure   PutPixel( MemBMPOffset : Longint; strength: Byte; Color : Colorref);assembler;//Strength:= 0 = transparent   //Strength:= 255 = opaque
var r,g,b:Byte;
 asm
  mov eax,MemBMPW // this section ...
  mov ebx,MemBMPh
  mul ebx
  mov ebx,MemBMPOffset
  cmp eax,ebx
  jng @done
  xor ebx,07fffffffh
  cmp ebx,000000000h
  jng @done      // ...only makes sure we are not operating outside the allowed memory area

  mov eax,DIBBits
  mov EDI,eax  // Bitmap memory address into EDI

  mov al,strength
  cmp al,255           // if the pixel is not to be faded we are just setting its new value
  jne @faded_pixel

  //Simply put pixel without fading to background color
  mov eax, MemBMPOffset
  mov ebx, 3        //24-bit color
  mul ebx
  add edi, eax

  //Pixel located
  Mov ebx,Color  // Store color values in variables

  mov al,r
  mov [edi+2],bl
  // red written
  mov al,g
  mov [edi+1],bh
  // green written

  shr ebx,16
  mov al,bl  //b

  mov al,b
  mov [edi],bl
  // blue written
  jmp @done

 @faded_pixel:
  //Put pixel faded to background color
  Mov ebx,Color  // Store color values in variables
  mov r,bl //r
  mov g,bh //g
  shr ebx,16
  mov b,bl  //b

  mov eax, MemBMPOffset
  mov ebx, 3        //24-bit color
  mul ebx
  add edi, eax

  //Pixel located

  mov eax,0
  mov ecx,0
  mov ebx,0        //Reset working area

  mov al,[edi+2]   // get old red
  mov cl,255
  mov bl,strength
  sub cl,bl
  mul cl          //ax = old * (255-strength)
  mov cx,ax       //Stored in ecx

  mov eax,0
  mov al,r
  mul bl          // ax = new*strength
  add ax,cx       // ax = old * (255-strength) + new*strength
  shr ax,8        // ax = (old * (255-strength) + new*strength)/255

  mov [edi+2],al
  // red written

  mov eax,0
  mov ecx,0
  mov ebx,0        //Reset working area

  mov al,[edi+1]   // get old green
  mov cl,255
  mov bl,strength
  sub cl,bl
  mul cl          //ax=old * (255-strength)
  mov ecx,eax     //Stored in ecx

  mov eax,0
  mov al,g
  mul bl          // ax = new*strength
  add ax,cx       // ax = old * (255-strength) + new*strength
  shr ax,8        // ax = (old * (255-strength) + new*strength)/255

  mov [edi+1],al
  // green written


  mov eax,0
  mov ecx,0
  mov ebx,0        //Reset working area

  mov al,[edi]     // get old blue
  mov cl,255
  mov bl,strength
  sub cl,bl
  mul cl          //ax=old * (255-strength)
  mov ecx,eax     //Stored in ecx

  mov eax,0
  mov al,b
  mul bl          // ax = new*strength
  add ax,cx       // ax = old * (255-strength) + new*strength
  shr ax,8        // ax = (old * (255-strength) + new*strength)/255

  mov [edi],al
  // blue written
@done:
 end;

var Anti_Alias_Gain                                 : Byte;

procedure  DrawSmoothLine( x1, y1, x2, y2 : Longint; color : Colorref);
// This is the one that gives us nice lines.
//
// It is derived from  bresenham's Line Drawing Algorithm, but not optimized at all.
// Feel free to do that work for me, and send me a copy of the new (assembler) code.
// As stated before, I would only be happy if I got e-letters about this code.
var XScope,
    YScope,
    XDir,
    YDir,
    LinearDeviance,
    Length,
    Counter : integer;
    VGA_Offset : Longint;
    r,g,b:Byte;
    AntiAliasStrength : Byte;
    EndPointIntensity : Byte;
begin
    LinearDeviance := 0;

    VGA_Offset := 0;

    XScope := (x2 - x1);
    YScope := (y2 - y1);

   if (XScope < 0) then
    begin
     XScope := abs(XScope);
     XDir := -1;
    end
   else
     XDir := 1;

   if (YScope < 0) then
    begin
     YScope := abs(YScope);
     YDir := -1 * MemBMPW;
    end
   else
     YDir := MemBMPW;

   VGA_Offset := x1 + (y1*MemBMPW);
   if (XScope > YScope) then
    begin {to increment X}
     Length := XScope;// + 1;
     LinearDeviance := XScope div 2;
     EndPointIntensity:=Round(Anti_Alias_Gain / 3 *Yscope/Xscope);
     for  counter := 0 to length do
      begin
       if Counter = 0 then
        begin
         PutPixel(VGA_Offset-Xdir-YDir,EndPointIntensity, Color );
         PutPixel(VGA_Offset-xDir,Anti_Alias_Gain div 2, Color );
        end;
       if Counter = length then
        begin
         PutPixel(VGA_Offset+Xdir+YDir,EndPointIntensity, Color );
         PutPixel(VGA_Offset+xDir,Anti_Alias_Gain div 2, Color );
        end;
       AntiAliasStrength:=Round(LinearDeviance / XScope*Anti_Alias_Gain);
       PutPixel(VGA_Offset,255, Color );
       PutPixel(VGA_Offset-Ydir,Anti_Alias_Gain-AntiAliasStrength, Color );
       PutPixel(VGA_Offset+Ydir,AntiAliasStrength, Color );
       LinearDeviance := (LinearDeviance + YScope);     {here's one}
       if (LinearDeviance >= XScope) then
        begin
         LinearDeviance := (LinearDeviance - XScope);              {-}
         VGA_Offset := VGA_Offset + YDir;                    {+}
        end;
       VGA_Offset := (VGA_Offset + XDir);  {+}
      end;
    end
   else          {increment Y instead}
    begin
     Length := YScope;// + 1;
     LinearDeviance := yScope div 2;
     EndPointIntensity:=Round(Anti_Alias_Gain / 3 *Xscope/Yscope);
     for counter:= 0 to length do
      begin
       if Counter = 0 then
        begin
         PutPixel(VGA_Offset-YDir-Xdir,EndPointIntensity, Color );
         PutPixel(VGA_Offset-YDir,Anti_Alias_Gain div 2, Color );
        end;
       if Counter = length then
        begin
         PutPixel(VGA_Offset+YDir+Xdir,EndPointIntensity, Color );
         PutPixel(VGA_Offset+YDir,Anti_Alias_Gain div 2, Color );
        end;
       AntiAliasStrength:=Round(LinearDeviance / YScope*Anti_Alias_Gain);
       PutPixel(VGA_Offset,255, Color );
       PutPixel(VGA_Offset-Xdir,Anti_Alias_Gain-AntiAliasStrength, Color );
       PutPixel(VGA_Offset+Xdir,AntiAliasStrength, Color );
       LinearDeviance := (LinearDeviance + XScope);            {+}
       if (LinearDeviance >= YScope) then
        begin {to big?}
         LinearDeviance := (LinearDeviance - YScope); {-}
         VGA_Offset := (VGA_Offset + XDir);                         {+}
        end;
       VGA_Offset := (VGA_Offset + YDir);                         {+}
      end;
    end;
end;




///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
/////////////////////////////      Main       /////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////



Const
  WM_POWERBROADCAST   = 536;


Procedure PutPixelBox(x1 , y1:Longint; Color :ColorRef; magn:Byte);
var a,b : DWord;
 begin
  for a := 0 to magn-1 do
   for b := 0 to magn-1 do
    PutPixelXY(x1+a,y1+b, Color);
 end;

var  Magnification_Factor                   : Byte;

Procedure Magnify(x1,y1:Longint);
var Color : ColorRef;
    x,y:Longint;
 begin
 if x1 > 400 - 1 - 50 div Magnification_Factor * 2  then x1 := 400 - 1 - 50 div Magnification_Factor * 2;
 if x1 < 1 + 50   div Magnification_Factor * 2  then x1 := 1+ 50    div Magnification_Factor * 2;
 if y1 > 400 - 1 - 50 div Magnification_Factor * 2  then y1 := 400 - 1 - 50 div Magnification_Factor * 2;
 if y1 < 1 + 50   div Magnification_Factor * 2  then y1 := 1+ 50    div Magnification_Factor * 2;
  for x := x1 to x1+100 div Magnification_Factor *2 do
   for y := y1 to y1+100 div Magnification_Factor *2 do
    begin
     Color:=GetpixelXY(x-100 div Magnification_Factor,y-100  div Magnification_Factor );
     PutPixelBox(400+(x*Magnification_Factor-(x1*Magnification_Factor)) ,100+ (y*Magnification_Factor-(y1*Magnification_Factor)), Color,Magnification_Factor);
    end;
 end;

Procedure UpdateCaption;
 begin
  Str(Anti_Alias_Gain/2.55:0:1,s);
  Str(Magnification_Factor,s1);
  s:='Anti-Aliasing Line Drawing Demo          -          Anti-Aliasing Strength: '+s+' % , Magnification Factor: '+s1+#0;
  SetWindowText(AppWnd,@s[1]);
 end;
Function Max(a,b:double):Double; begin  if a> b then  Result := a else  Result := b; end;
Function Min(a,b:double):Double; begin  if a< b then  Result := a else  Result := b; end;

Function  MyWndproc conv arg_stdcall(Wnd:hwnd;mess:uint;wParam:WPARAM;lParam:LPARAM):longint;// windowproc
var xpos , ypos : Longint;
 begin
 {$ifdef DBG}
//  writeln(Mess);
 {$endif}
  case Mess of
    WM_PAINT , WM_POWERBROADCAST, WM_CTLCOLORBTN:
     begin
      BitBlt(DC,WrkArea.left,WrkArea.top,WrkArea.right-WrkArea.left+1,WrkArea.bottom-WrkArea.top+1,MemDC,0,0,SRCCOPY);
      DefWindowProc(Wnd,mess,wParam,lParam);
     end;
    WM_HSCROLL,WM_VSCROLL:
     begin
       if (lParam = Track_Mag) then
        case LOWORD(wParam) of
         TB_ThumbTRACK:
           Magnification_Factor := Sendmessage(Track_Mag,TBM_GETPOS,0,0);
         TB_ENDTRACK:
          begin
           Magnification_Factor := Sendmessage(Track_Mag,TBM_GETPOS,0,0);
           SendMessage(AppWnd,WM_COMMAND,13,0);
          end;
        end;

       if (lParam = Track_AAG) then
        case LOWORD(wParam) of
         TB_ThumbTRACK:
           Anti_Alias_Gain := Sendmessage(Track_AAG,TBM_GETPOS,0,0);
         TB_ENDTRACK:
          begin
           Anti_Alias_Gain := Sendmessage(Track_AAG,TBM_GETPOS,0,0);
           SendMessage(AppWnd,WM_COMMAND,13,0);
          end;
        end;
      UpdateCaption;
     end;
    WM_MOUSEMOVE:
     begin
      xPos := LOWORD(lParam)-WrkArea.left;
      yPos := HiWORD(lParam)-WrkArea.top;
      Magnify(xPos,yPos);
      SendMessage(Wnd,WM_PAINT,0,0);
     end;
    WM_COMMAND:
     begin
      case LoWord(wParam) of
       12: SendMessage(Wnd,WM_CLOSE,0,0);
       13:
        begin
         PatBlt(MemDC,0,0,rect.right,rect.bottom,BLACKNESS);
          a:=1;
         for a :=0 to 400  do
          for q :=0 to 400  do
           PutPixelXY ( a ,q  ,RGB( Round(Max(0,Min(255,150 * sqr(sqr((400-a)  / 400)) + 250 *sqr(sqr( (400-q) / 400)))))     ,
                                    Round(Max(0,Min(255,250 * sqr(sqr((400-q)  / 400)) + 150 *sqr(sqr( q / 400))))),
                                    Round(150 *sqr( a  / 400)))   );
         SendMessage(Wnd,WM_PAINT,0,0);

         for a := 0 to 29 do
           DrawSmoothLine    (    200 + Round(Cos(2*pi*a/30)*50),
                                  200 - Round(Sin(2*pi*a/30)*50),
                                  200 + Round(Cos(2*pi*a/30)*150),
                                  200 - Round(Sin(2*pi*a/30)*150),RGB(Round(255-Round(sqr(sin(a/30*2*pi/3)*255)))     ,
                                                                      Round(Round(sqr(sin(a/30*2*pi/3)*255))) ,
                                                                      Round(255-Round(sin(a/30*2*pi/3)*255))));
         for a := 0 to 29 do
           DrawSmoothLine    (    200 + Round(Cos(2*pi*(a+0.5)/30)*50),
                                  200 - Round(Sin(2*pi*(a+0.5)/30)*50),
                                  200 + Round(Cos(2*pi*(a+0.5)/30)*150),
                                  200 - Round(Sin(2*pi*(a+0.5)/30)*150),rgb(255*Round(a/27)
                                                                        ,255*Round(a/27)
                                                                        ,255*Round(a/27)  ));
         for a := 1 to 36 do
          begin
           DrawSmoothLine    (  200 + Round(Cos(2*pi*a/12)*(110 +a)),
                                200 - Round(Sin(2*pi*a/12)*(110 +a)),
                                200 + Round(Cos(2*pi*(a-1)/12)*(110+(a-1))),
                                200 - Round(Sin(2*pi*(a-1)/12)*(110+(a-1))),RGB(255-Round(255 * a      / 100)      ,
                                                                      Round(128 * (10-a*2) / 100) ,
                                                                      Round(128* (a*3)  / 100)) );

          end;
         for a := 1 to 300 do
          begin
           DrawSmoothLine    (  200 + Round(Cos(2*pi*a/300)*180),
                                200 - Round(Sin(2*pi*a/300)*180),
                                200 + Round(Cos(2*pi*(a-1)/300)*180),
                                200 - Round(Sin(2*pi*(a-1)/300)*180),RGB(255-Round(255 * a      / 100)      ,
                                                                      Round(128 * (10-a*2) / 100) ,
                                                                      Round(128* (a*3)  / 100)) );
          end;
          DrawSmoothLine(10,10,390,10,RGB(255,255,255));
          DrawSmoothLine(390,10,390,390,RGB(255,255,255));
          DrawSmoothLine(390,390,10,390,RGB(255,255,255));
          DrawSmoothLine(10,390,10,10,RGB(255,255,255));
          SendMessage(Wnd,WM_PAINT,0,0);
        end;
       201:ShellExecute(0,'open','http://www.widar.f2s.com',nil,nil,SW_SHOW );
      end;
     end;
    WM_DESTROY: PostQuitMessage(0);

    WM_SIZE :
     begin
      GetClientRect(AppWnd,rect);
      If Rect.right < 50 then
       begin
        GetWindowRect(AppWnd,Rect);
        MoveWindow( Appwnd, Rect.left, Rect.top, 50, rect.bottom-rect.top, true);
        exit;
       end;
      GetClientRect(AppWnd, WrkArea);
      WrkArea.left := 90;
      WrkArea.top  := 75;
      WrkArea.Right  := WrkArea.Right  - 4;
      WrkArea.bottom := WrkArea.bottom - 4;
     end;

    WM_CLOSE:
      DestroyWindow(Wnd);
  else
    Result:=DefWindowProc(Wnd,mess,wParam,lParam);
  end;
 end;

Procedure Allocateacceleratortable;
 begin
  Accelerators[0].fVirt:=FVIRTKEY;
  Accelerators[0].key:=VK_ESCAPE;
  Accelerators[0].cmd:=12;

  Accelerators[1].fVirt:=FVIRTKEY;
  Accelerators[1].key:=VK_RETURN;
  Accelerators[1].cmd:=13;
  Acceleratortable := CreateAcceleratorTable(Accelerators,2);
 end;

function CreateWndClass(ClassName: PChar; WndProc: Pointer): Boolean;
var
  wc : TWndClass;
begin
  FillChar(wc, SizeOf(wc), 0);
  wc.style := CS_HREDRAW + CS_VREDRAW;
  wc.lpfnWndProc := WndProc;
  wc.cbClsExtra := 0;
  wc.cbWndExtra := 0;
  wc.hInstance := System.hInstance;
  wc.hIcon := LoadIcon(THandle(nil), IDI_APPLICATION);
  wc.hCursor := LoadCursor(THandle(nil), IDC_ARROW);
  wc.hbrBackGround := COLOR_WINDOW;
  wc.lpszMenuName := nil;
  wc.lpszClassName := ClassName;
  Result := RegisterClass(wc) <> 0;
end;

begin
 InitCommonControls;

 CreateWndClass('Main',@MyWndProc);
 AppWnd       :=  CreateWindowEx(WS_EX_TOOLWINDOW, 'Main'        , 'Test 0%'             , WS_CAPTION +WS_SYSMENU                                ,0   ,100,700,500,0     , 0,   hinstance,nil);

 TextLabel    :=  CreateWindow(                    'STATIC'      , 'Anti-Aliasing Strength'   , WS_VISIBLE+WS_CHILD+SS_RIGHT                          , 0  ,2  ,85 ,30 ,AppWnd, 0,   hinstance,nil);
 Sendmessage(TextLabel,WM_SetFont,GetStockObject(ANSI_VAR_FONT),1);
 TextLabel    :=  CreateWindow(                    'STATIC'      , 'Magnification Factor', WS_VISIBLE+WS_CHILD+SS_RIGHT                          , 0  ,45 ,85 ,30 ,AppWnd, 0,   hinstance,nil);
 Sendmessage(TextLabel,WM_SetFont,GetStockObject(ANSI_VAR_FONT),1);

 Track_AAG    :=  CreateWindow(                    TRACKBAR_CLASS, ''                    , WS_VISIBLE+WS_CHILD+TBS_HORZ+TBS_BOTTOM+TBS_AUTOTICKS , 90 ,0  ,600,35 ,AppWnd, 1111,hinstance,nil);
 Track_Mag    :=  CreateWindow(                    TRACKBAR_CLASS, ''                    , WS_VISIBLE+WS_CHILD+TBS_HORZ+TBS_TOP+TBS_AUTOTICKS    , 90 ,38 ,600,35 ,AppWnd, 1112,hinstance,nil);

 SendMessage(Track_AAG,TBM_SETRANGE,1,MAKELONG(0,255));
 SendMessage(Track_AAG,TBM_SETTICFREQ,16,1);

 SendMessage(Track_AAG,TBM_SETPAGESIZE,0,16);
 SendMessage(Track_Mag,TBM_SETRANGE,1,MAKELONG(1,8));

 SendMessage(Track_AAG,TBM_SETPOS,1,160);
 SendMessage(Track_Mag,TBM_SETPOS,1,3);


 Anti_Alias_Gain      := Sendmessage(Track_AAG,TBM_GETPOS,0,0);
 Magnification_Factor := Sendmessage(Track_Mag,TBM_GETPOS,0,0);
 UpdateCaption;

 Btn_Banner   :=  CreateWindowEx(WS_EX_CLIENTEDGE  ,'BUTTON' ,''     ,WS_CHILD + WS_VISIBLE + BS_BITMAP  ,2,75,84,402,AppWnd,201,hinstance,nil);
 Banner_BMP := LoadBitmap(hinstance,MAKEINTRESOURCE(1001 ));
 SendMessage(Btn_Banner , BM_SETIMAGE , IMAGE_BITMAP , Banner_BMP );

 InitMemory;
 SendMessage(AppWnd,WM_COMMAND,13,0);
 ShowWindow(AppWnd,SW_SHOW);

 Allocateacceleratortable;

 while GetMessage(Msg, 0, 0, 0) do
 begin
   TranslateAccelerator(AppWnd,Acceleratortable,Msg);
   DispatchMessage(Msg);
 end;
 DeleteDC(MemDC);
 DeleteObject(MemBMP);
 ReleaseDC(AppWnd,DC);
 {$ifdef DBG}
  DoneWinCrt;
 {$endif}
end.